//
// Copyright (c) 2001-2019 by PDFTron Systems Inc. All Rights Reserved.
//

using System;
using System.Collections.Generic;

using pdftron;
using pdftron.Common;
using pdftron.Filters;
using pdftron.SDF;
using pdftron.PDF;

using XSet = System.Collections.Generic.List<int>;

namespace ElementEditTestCS
{
	class Class1
	{
		private static pdftron.PDFNetLoader pdfNetLoader = pdftron.PDFNetLoader.Instance();
		static Class1() {}

        static string layerToRemove = "Text Layer";

		static void ProcessElements(ElementReader reader, ElementWriter writer, XSet visited)
		{
			Element element;
			while ((element = reader.Next()) != null) // Read page contents
			{
				switch (element.GetType())
				{
					case Element.Type.e_form:
						{
                            Obj form_obj = element.GetXObject();
                            var num = form_obj.GetObjNum();
                            Obj oc = form_obj.FindObj("OC");
                            if(oc != null && oc.IsDict())
                            {
                                Obj ocName = oc.FindObj("Name");
                                if (ocName != null && ocName.IsString())
                                {
                                    string name = ocName.GetAsPDFText();
                                    if (name == layerToRemove)
                                    {
                                        continue;
                                    }
                                }
                            }
                            writer.WriteElement(element);
							break;
						}
                    default:
                        writer.WriteElement(element);
                        break;
				}
			}
		}

        static void EraseNamedObjectInArray(Obj array, string name)
        {
            if (array == null || !array.IsArray()) return;

            int i = 0;
            for (; i < array.Size(); ++i)
            {
                var layer = array.GetAt(i);
                if (layer.FindObj("Name").GetAsPDFText().CompareTo(name) == 0) break;
            }
            if (i < array.Size())
            {
                array.EraseAt(i);
            }
        }
		
		[STAThread]
		static void Main(string[] args)
		{
			PDFNet.Initialize();

			// Relative path to the folder containing test files.
			string input_path = "../../../../TestFiles/";
			string output_path = "../../../../TestFiles/Output/";

			try
			{
				Console.WriteLine("Opening the input file...");
                using (PDFDoc doc = new PDFDoc(output_path + "pdf_layers.pdf"))
				{
					doc.InitSecurityHandler();

					ElementWriter writer = new ElementWriter();
					ElementReader reader = new ElementReader();
					XSet visited = new XSet();

					PageIterator itr = doc.GetPageIterator();

					while (itr.HasNext())
					{
						try
						{
							Page page = itr.Current();
							visited.Add(page.GetSDFObj().GetObjNum());
							reader.Begin(page);
							writer.Begin(page, ElementWriter.WriteMode.e_replacement, false, true, page.GetResourceDict());
                            ProcessElements(reader, writer, visited);
							writer.End();
							reader.End();
							itr.Next();
						}
						catch (PDFNetException e)
						{
							Console.WriteLine(e.Message);
						}
					}

                    var ocgConfig = doc.GetOCGConfig();
                    var order = ocgConfig.GetOrder();
                    EraseNamedObjectInArray(order, layerToRemove);
                    var ocgs = doc.GetOCGs();
                    EraseNamedObjectInArray(ocgs, layerToRemove);

                    doc.Save(output_path + "pdf_layer-removed.pdf", SDFDoc.SaveOptions.e_linearized);
				}
			}
			catch (PDFNetException e)
			{
				Console.WriteLine(e.Message);
			}

		}
	}
}
